import {
  select,
  csv,
  scaleLinear,
  max,
  scaleBand,
  axisLeft,
  axisBottom,
  format,
  tip,
  scaleOrdinal,
  mouse,
} from 'd3';

const titleText =
  'Top food types in top cities without Boston';
const xAxisLabelText =
  'Scores';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = (data) => {
  //const xValue = d => +d['count'];
  const yValue = (d) => d.city;
  //const cValue = d => d.food;
  const margin = {
    top: 50,
    right: 40,
    bottom: 77,
    left: 120,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

	const keys = ['scores_teaching', 'scores_research', 'scores_citations', 'scores_industry_income', 'scores_international_outlook']

  const food = data.columns.slice(1);

  const city = d3.map(data, (d) => d.city);
  //console.log(city)
	
  const stackedData = d3.stack().keys(food)(data);
  //console.log(food)
  //console.log(stackedData);
  const colorScale = [
    '#ede898',
    '#f7d09c',
    '#f7b19c',
    '#9cdff7',
    '#cdf79c',
    '#bf6b7c',
    '#9cf7ce',
    '#ff9ee4',
    '#7dfff6',
    '#fc8383',
  ];
  const color = scaleOrdinal()
    .domain(food)
    .range(colorScale);

  //console.log(color)

  const xScale = scaleLinear()
    .domain([0, 170])
    .range([0, innerWidth]);

  const yScale = scaleBand()
    .domain(city)
    .range([0, innerHeight])
    .padding(0.1);

  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`
    );

  const xAxis = axisBottom(xScale).tickSize(-innerHeight);

  g.append('g')
    .call(axisLeft(yScale))
    .selectAll('.domain, .tick line')
    .remove();

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG.select('.domain').remove();

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 65)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabelText);

  // add the tool tip to the body
  const tooltip = select('body').append('div').attr('class', 'toolTip');
  
  tooltip
    .append('rect')
    .attr('width', 60)
    .attr('height', 20)
    .attr('fill', 'none')
    .style('opacity', 0.5);

  tooltip
    .append('text')
    .attr('x', 30)
    .attr('dy', '1.2em')
    .style('text-anchor', 'middle')
    .attr('font-size', '12px')
    .attr('font-weight', 'bold');
  
  g.append('g')
    .selectAll('g')
    // Enter in the stack data = loop key per key = group per group
    .data(stackedData)
    .enter()
    .append('g')
    .attr('fill', (d) => color(d.key))
    .selectAll('rect')
    // enter a second time = loop subgroup per subgroup to add all rectangles
    .data((d) => d)
    .enter()
    .append('rect')
    .attr('y', (d) => yScale(d.data.city))
    .attr('x', (d) => xScale(d[0]))
    .attr('width', (d) => xScale(d[1]) - xScale(d[0]))
    .attr('height', yScale.bandwidth())
    .on('mouseover', function (event, d) {
      d3.select(this).attr('opacity', 0.5);
    })
    .on('mousemove', function (event, d) {
    	// can just leave this all in the mouse move event
      const foodtype = d3.select(this.parentNode).datum()
        .key;
      const foodcount = d.data[foodtype];
      tooltip
        .style('left', event.pageX - 50 + 'px')
        .style('top', event.pageY - 70 + 'px')
        .style('display', 'inline-block')
        .html(foodtype + ' food: ' + foodcount);
    })
    .on('mouseout', function (d) {
      d3.select(this).attr('opacity', 1);
      tooltip.style('display', 'none');
    });

  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text(titleText);

  g.append('text')
    .attr('text-anchor', 'right')
    .attr('x', 670)
    .attr('y', 100)
    .style('font-size', '20px')
    .style('font-weight', 'bold')
    .text('Food types');

  g.selectAll('legendsqr')
    .data(food)
    .enter()
    .append('rect')
    .attr('x', 660)
    .attr('y', function (d, i) {
      return 110 + i * 26;
    })
    .attr('width', 15)
    .attr('height', 15)
    .style('fill', function (d) {
      return color(d);
    });

  g.selectAll('legendLabels')
    .data(food)
    .enter()
    .append('text')
    .attr('x', 700)
    .attr('y', function (d, i) {
      return 118 + i * 26;
    })
    .style('fill', function (d) {
      return color(d);
    })
    .text(function (d) {
      return d;
    })
    .attr('text-anchor', 'left')
    .style('alignment-baseline', 'middle')
    .style('font-size', '18px')
    .style('font-weight', 'bold');
};

csv('data.csv').then((data) => {
  render(data);
});

//Boston,235,134,64,44,56,29,58,15,45,23
